/***********************************************/
/* Software Created By CKJ Vincent Cedric      */
/* This code is the property of Vincent Cedric */
/* You can't copy or use this code without     */
/* according of the author (GPL License).	   */
/***********************************************/
// tracker Addons to convert Image to Icon
// and Add the Icon to the selected File
#include <Application.h>
#include <Alert.h>
#include <Entry.h>
#include <TrackerAddon.h>
#include <TranslationUtils.h>
#include <Path.h>
#include <Bitmap.h>
#include <View.h>
#include <Node.h>
#include <NodeInfo.h>

// definition des fonctions
status_t	ConvertToIcon(BEntry *entry);
void		ScaleToFit(uint16 &width,uint16 &height);
void		FillTransparent(BBitmap *icon);
void		ScaleBitmapToIcon(BNodeInfo *nodeInfo,BBitmap *bitmap,icon_size which,uint16 width,uint16 height);


/**** fonction appelé par le tracker ****/
void process_refs(entry_ref dir_ref, BMessage *msg, void *reserved)
{
	status_t	state = B_OK;
	entry_ref	fileRef;
	BEntry		entry;
	int32		index;
	
	// parcourir le message pour trouver
	// les fichiers a convertir
	index=0;
	while(msg->FindRef("refs",index,&fileRef)==B_OK)
	{
		// verifier la validité du fichier
		entry.SetTo(&fileRef);
		if(entry.InitCheck()==B_OK)
			state &= ConvertToIcon(&entry);
			
		// passer au fichier suivant
		index++;
	}
}

/**** prevenir que c'est un tracker-addon ****/
int main(void)
{
	BApplication		errorApp("application/x-vnd.CKJ-Tracker-Addons-IconToImage");

	// lancer le message d'erreur
	(new BAlert("","This is a Tracker Add-On.\nYou can't start-it, use tracker Add-Ons menu.","Ok"))->Go();

	return (0);
}

/**** convertion de l'image en icon ****/
status_t ConvertToIcon(BEntry *entry)
{
	// quand on arrive ici on a un fichier valide
	// il faut essayer de convertir les données du fichier en image
	BBitmap		*bitmap = NULL;
	BPath		filePath;
	
	filePath.SetTo(entry);
	if(filePath.InitCheck()!=B_OK)
		return B_ERROR;
	
	// essayer de recuperer l'image
	bitmap = BTranslationUtils::GetBitmapFile(filePath.Path());
	if(bitmap==NULL)
		return B_ERROR;
	
	// initialiser le NodeInfo
	BNode		node(entry);
	BNodeInfo	nodeInfo(&node);

	if(nodeInfo.InitCheck()!=B_OK)
		return B_ERROR;
		
	// recalibrer la taille
	uint16		size_x,size_y;

	size_x = (uint16)((bitmap->Bounds()).Width());
	size_y = (uint16)((bitmap->Bounds()).Height());
	ScaleToFit(size_x,size_y);
	
	// creation de l'icon 32*32
	ScaleBitmapToIcon(&nodeInfo,bitmap,B_LARGE_ICON,size_x,size_y);
	
	// creation de l'icon 16*16
	ScaleBitmapToIcon(&nodeInfo,bitmap,B_MINI_ICON,size_x/2,size_y/2);
	
	// liberer la memoire
	delete bitmap;
	
	// ok
	return B_OK;
}

/**** redimentionner en fonction de la taille d'origine ****/
void ScaleToFit(uint16 &width,uint16 &height)
{
	float		step = 0;
	
	// on fonction du coté le plus grand
	if(width>height)
	{
		step = width / 31;
		width = 31;
		height = (uint16)(height / step);
	}
	else
	{
		step = height / 31;
		height = 31;
		width = (uint16)(width / step);
	}
}

/**** remplir avec la couleur transparente ****/
void FillTransparent(BBitmap *icon)
{
	int32	size;

	size = (int32)(((icon->Bounds()).Width()+1) * ((icon->Bounds()).Height()+1));
	for(int32 pixel=0;pixel<size;pixel++)
		*((int8 *)(icon->Bits()) + pixel) = B_TRANSPARENT_MAGIC_CMAP8;
}

/***** dessiner l'image au bon format ****/
void ScaleBitmapToIcon(BNodeInfo *nodeInfo,BBitmap *bitmap,icon_size which,uint16 width,uint16 height)
{
	// on va maintenant assigner l'icon au fichier
	BView		*offView = NULL;
	BBitmap		*icon = NULL;

	// creer l'icon
	offView = new BView(BRect(0,0,which-1,which-1),"off-view",B_FOLLOW_NONE,B_WILL_DRAW);	
	icon = new BBitmap(BRect(0,0,which-1,which-1),B_CMAP8,true,true);

	// fill with transparent color
	FillTransparent(icon);

	// Draw the Icon
	icon->Lock();
	icon->AddChild(offView);
	offView->DrawBitmapAsync(bitmap,BRect(0,0,width,height));
	offView->Sync();
	icon->Unlock();

	// Set to node
	nodeInfo->SetIcon(icon,which);

	// liberer la memoire
	icon->Lock();
	icon->RemoveChild(offView);
	icon->Unlock();
	delete offView;
	delete icon;
}